local utf8 = require 'utf8'

local my_utf8 = {}

function my_utf8.offset(s, pos1)
  if pos1 == 1 then return 1 end
  local result = utf8.offset(s, pos1)
  if result == nil then
    assert(false, ('my_utf8.offset(%d) called on a string of length %d (byte size %d); this is likely a failure to handle utf8\n\n^%s$\n'):format(pos1, utf8.len(s), #s, s))
  end
  return result
end

function my_utf8.match_at(s, pos, pat)
  local start_offset = my_utf8.offset(s, pos)
  local end_offset = my_utf8.offset(s, pos+1)
  assert(end_offset > start_offset, ('end_offset %d not > start_offset %d'):format(end_offset, start_offset))
  local curr = s:sub(start_offset, end_offset-1)
  return curr:match(pat)
end

-- optionally take a range to check within
-- if provided, caller is responsible for the validity of the range
function my_utf8.pos(s, byte_idx, start_pos, max_pos)
  for pos = start_pos or 1, max_pos or utf8.len(s) do
    local curr_byte = utf8.offset(s, pos)
    if curr_byte == byte_idx then
      return pos
    end
  end
  assert(false, 'ill-formed utf-8 string')
end

-- splice out a substring between spos and epos (both inclusive)
function my_utf8.substring(str, spos, epos)
  local start_offset = my_utf8.offset(str, spos)
  local end_offset = my_utf8.offset(str, epos+1)
  assert(end_offset >= start_offset, ('end_offset %d not >= start_offset %d'):format(end_offset, start_offset))
  return str:sub(start_offset, end_offset-1)
end

-- create a new iterator for s which provides the index and UTF-8 bytes corresponding to each codepoint
function my_utf8.chars(s, startpos)
  local next_pos = startpos or 1  -- in code points
  local next_offset = utf8.offset(s, next_pos)  -- in bytes
  return function()
    assert(next_offset)  -- never call the iterator after it returns nil
    local curr_pos = next_pos
    next_pos = next_pos+1
    local curr_offset = next_offset
    next_offset = utf8.offset(s, 2, next_offset)
    if next_offset == nil then return end
    local curr_char = s:sub(curr_offset, next_offset-1)
    return curr_pos, curr_char
  end
end

return my_utf8
